/* version: r1781 */
var Scrutari = {};
Scrutari.log = function (msg) {
    if ((console) && (console.log)) {
        console.log(msg);
    }
};
Scrutari.logError = function (error) {
    var errorMessage = "Scrutari Request Error [key = " + error.key + " | parameter = " + error.parameter;
    if (error.hasOwnProperty("value")) {
        errorMessage += " | value = " + error.value;
    }
    if (error.hasOwnProperty("array")) {
        errorMessage += " | array = (";
        for(var i = 0; i < error.array.length; i++) {
            if (i > 0) {
                errorMessage += ";";
            }
            var obj = error.array[i];
            errorMessage += obj.key;
            if (obj.hasOwnProperty("value")) {
                errorMessage += "=" + obj.value;
            }
        }
        errorMessage += ")";
    }
    errorMessage += "}";
    Scrutari.log(errorMessage);
};
Scrutari.convert = function (jqArgument) {
    if (jqArgument.jquery) {
        return jqArgument;
    } else {
        return $(jqArgument);
    }
};
Scrutari.exists = function (jqArgument) {
    return Scrutari.convert(jqArgument).length > 0;
};
 Scrutari.escape = function (text) {
     var result = "";
    for(var i = 0; i < text.length; i++) {
        carac = text.charAt(i);
        switch (carac) {
            case '&':
                result += "&amp;";
                break;
            case '"':
                result += "&quot;";
                break;
            case '<':
                result += "&lt;";
                break;
            case '>':
                result += "&gt;";
                break;
            case '\'':
                result += "&#x27;";
                break;
            default:
                result += carac;
        }
    }
    return result;
 };
Scrutari.Config = function (name, engineUrl, langUi, origin) {
    this.name = name;
    this.engineUrl = engineUrl;
    this.langUi = langUi;
    this.origin = origin;
    this.ficheFields = "";
    this.plageLength = 50;
    this.groupSortFunction = Scrutari.Config.FicheCountGroupSortFunction;
};
Scrutari.Config.prototype.getName = function () {
    return this.name;
};
Scrutari.Config.prototype.getJsonUrl = function () {
    return this.engineUrl + "json";
};
Scrutari.Config.prototype.getLangUi = function () {
    return this.langUi;
};
Scrutari.Config.prototype.getOrigin = function () {
    return this.origin;
};
Scrutari.Config.prototype.getPlageLength = function () {
    return this.plageLength;
};
Scrutari.Config.prototype.setPlageLength = function (plageLength) {
    this.plageLength = plageLength;
};
Scrutari.Config.prototype.getOdsUrl = function (qId) {
    return this.engineUrl + "export/" +  "result_" + qId + "_" + this.langUi + ".ods";
};
Scrutari.Config.prototype.getAtomUrl = function (qId) {
    var date = new Date();
    var dateString = date.getFullYear() + "-";
    var mois = date.getMonth() + 1;
    if (mois < 10) {
        dateString += "0";
    }
    dateString += mois;
    dateString += "-";
    var jour = date.getDate();
    if (jour < 10) {
        dateString += "0";
    }
    dateString += jour;
    return this.engineUrl + "feed/" + "fiches_" + this.langUi + ".atom?qid=" + qId + "&all=" + dateString;
};
Scrutari.Config.prototype.getPermalinkUrl = function (qId, permalinkPattern) {
    var permalink = permalinkPattern.replace("$LANG", this.langUi);
    permalink = permalink.replace("$QID", qId);
    return permalink;
};
Scrutari.Config.prototype.getFicheFields = function () {
    return this.ficheFields;
};
Scrutari.Config.prototype.setFicheFields = function (ficheFields) {
    this.ficheFields = ficheFields;
};
Scrutari.Config.prototype.setGroupSortFunction = function (groupSortFunction) {
    this.groupSortFunction = groupSortFunction;
};
Scrutari.Config.prototype.getGroupSortFunction = function () {
    return this.groupSortFunction;
};
Scrutari.Config.FicheCountGroupSortFunction = function (group1, group2) {
    var count1 = group1.ficheCount;
    var count2 = group2.ficheCount;
    if (count1 > count2) {
        return -1;
    } else if (count1 < count2) {
        return 1;
    } else {
        var rank1 = group1.category.rank;
        var rank2 = group1.category.rank;
        if (rank1 < rank2) {
            return -1;
        } else if (rank1 > rank2) {
            return 1;
        } else {
            return 0;
        }
    }
};
Scrutari.Ajax = {};
Scrutari.Ajax.loadBaseArray = function (baseArrayCallback, apiConfig, requestParameters) {
    if (!requestParameters) {
        requestParameters = new Object();
    }
    requestParameters.type = "base";
    Scrutari.Ajax.check(requestParameters, "langui", apiConfig.getLangUi());
    Scrutari.Ajax.check(requestParameters, "warnings", 1);
    Scrutari.Ajax.check(requestParameters, "version", 2);
    $.ajax({
        url: apiConfig.getJsonUrl(),
        dataType: "jsonp",
        data: requestParameters,
        success: function (data, textStatus) {
            Scrutari.Ajax.success(data, "baseArray", baseArrayCallback);
        }
    });
};
Scrutari.Ajax.loadCorpusArray = function (corpusArrayCallback, apiConfig, requestParameters) {
    if (!requestParameters) {
        requestParameters = new Object();
    }
    requestParameters.type = "corpus";
    Scrutari.Ajax.check(requestParameters, "langui", apiConfig.getLangUi());
    Scrutari.Ajax.check(requestParameters, "warnings", 1);
    Scrutari.Ajax.check(requestParameters, "version", 2);
    $.ajax({
        url: apiConfig.getJsonUrl(),
        dataType: "jsonp",
        data: requestParameters,
        success: function (data, textStatus) {
            Scrutari.Ajax.success(data, "corpusArray", corpusArrayCallback);
        }
    });
};
Scrutari.Ajax.loadThesaurusArray = function (thesaurusArrayCallback, apiConfig, requestParameters) {
    if (!requestParameters) {
        requestParameters = new Object();
    }
    requestParameters.type = "thesaurus";
    Scrutari.Ajax.check(requestParameters, "langui", apiConfig.getLangUi());
    Scrutari.Ajax.check(requestParameters, "warnings", 1);
    Scrutari.Ajax.check(requestParameters, "version", 2);
    $.ajax({
        url: apiConfig.getJsonUrl(),
        dataType: "jsonp",
        data: requestParameters,
        success: function (data, textStatus) {
            Scrutari.Ajax.success(data, "thesaurusArray", thesaurusArrayCallback);
        }
    });
};
Scrutari.Ajax.loadMotcleArray = function (motcleArrayCallback, apiConfig, requestParameters) {
    if (!requestParameters) {
        requestParameters = new Object();
    }
    requestParameters.type = "motcle";
    Scrutari.Ajax.check(requestParameters, "motclefields", "motcleid,libelles");
    Scrutari.Ajax.check(requestParameters, "langui", apiConfig.getLangUi());
    Scrutari.Ajax.check(requestParameters, "warnings", 1);
    Scrutari.Ajax.check(requestParameters, "version", 2);
    $.ajax({
        url: apiConfig.getJsonUrl(),
        dataType: "jsonp",
        data: requestParameters,
        success: function (data, textStatus) {
            Scrutari.Ajax.success(data, "motcleArray", motcleArrayCallback);
        }
    });
};
Scrutari.Ajax.loadFicheArray = function (ficheArrayCallback, apiConfig, requestParameters) {
    if (!requestParameters) {
        requestParameters = new Object();
    }
    requestParameters.type = "fiche";
    Scrutari.Ajax.check(requestParameters, "fichefields", "titre");
    Scrutari.Ajax.check(requestParameters, "langui", apiConfig.getLangUi());
    Scrutari.Ajax.check(requestParameters, "warnings", 1);
    Scrutari.Ajax.check(requestParameters, "version", 2);
    $.ajax({
        url: apiConfig.getJsonUrl(),
        dataType: "jsonp",
        data: requestParameters,
        success: function (data, textStatus) {
            Scrutari.Ajax.success(data, "ficheArray", ficheArrayCallback);
        }
    });
};
Scrutari.Ajax.loadCategoryArray = function (categoryArrayCallback, apiConfig, requestParameters) {
    if (!requestParameters) {
        requestParameters = new Object();
    }
    requestParameters.type = "category";
    Scrutari.Ajax.check(requestParameters, "langui", apiConfig.getLangUi());
    Scrutari.Ajax.check(requestParameters, "warnings", 1);
    Scrutari.Ajax.check(requestParameters, "version", 2);
    $.ajax({
        url: apiConfig.getJsonUrl(),
        dataType: "jsonp",
        data: requestParameters,
        success: function (data, textStatus) {
            Scrutari.Ajax.success(data, "categoryArray", categoryArrayCallback);
        }
    });
};
Scrutari.Ajax.loadEngineInfo = function (engineInfoCallback, apiConfig, requestParameters) {
    if (!requestParameters) {
        requestParameters = new Object();
    }
    requestParameters.type = "engine";
    Scrutari.Ajax.check(requestParameters, "data", "all");
    Scrutari.Ajax.check(requestParameters, "langui", apiConfig.getLangUi());
    Scrutari.Ajax.check(requestParameters, "warnings", 1);
    Scrutari.Ajax.check(requestParameters, "version", 1);
    $.ajax({
        url: apiConfig.getJsonUrl(),
        dataType: "jsonp",
        data: requestParameters,
        success: function (data, textStatus) {
            Scrutari.Ajax.success(data, "engineInfo", engineInfoCallback);
        }
    });
};
Scrutari.Ajax.loadFicheSearchResult = function (ficheSearchResultCallback, apiConfig, requestParameters, apiErrorCallback) {
    var defaultFicheFields = apiConfig.getFicheFields();
    if (!defaultFicheFields) {
        defaultFicheFields = "codecorpus,mtitre,msoustitre,mattrs_all,mcomplements,annee,href,icon";
    }
    requestParameters.type = "q-fiche";
    Scrutari.Ajax.check(requestParameters, "langui", apiConfig.getLangUi());
    Scrutari.Ajax.check(requestParameters, "fichefields", defaultFicheFields);
    Scrutari.Ajax.check(requestParameters, "motclefields", "mlibelles");
    Scrutari.Ajax.check(requestParameters, "q-mode", "intersection");
    Scrutari.Ajax.check(requestParameters, "origin", apiConfig.getOrigin());
    Scrutari.Ajax.check(requestParameters, "warnings", 1);
    Scrutari.Ajax.check(requestParameters, "version", 2);
    Scrutari.Ajax.check(requestParameters, "start", 1);
    Scrutari.Ajax.check(requestParameters, "limit", apiConfig.getPlageLength() * 2);
    Scrutari.Ajax.check(requestParameters, "starttype", "in_all");
    $.ajax({
        url: apiConfig.getJsonUrl(),
        dataType: "jsonp",
        data: requestParameters,
        success: function (data, textStatus) {
            Scrutari.Ajax.success(data, "ficheSearchResult", ficheSearchResultCallback, apiErrorCallback);
        }
    });
};
Scrutari.Ajax.loadExistingFicheSearchResult = function (existingFicheSearchResultCallback, apiConfig, requestParameters) {
    var defaultFicheFields = apiConfig.getFicheFields();
    if (!defaultFicheFields) {
        defaultFicheFields = "codecorpus,mtitre,msoustitre,mattrs_all,mcomplements,annee,href,icon";
    }
    requestParameters.type = "q-fiche";
    Scrutari.Ajax.check(requestParameters, "langui", apiConfig.getLangUi());
    Scrutari.Ajax.check(requestParameters, "fichefields", defaultFicheFields);
    Scrutari.Ajax.check(requestParameters, "motclefields", "");
    Scrutari.Ajax.check(requestParameters, "warnings", 1);
    Scrutari.Ajax.check(requestParameters, "version", 2);
    Scrutari.Ajax.check(requestParameters, "start", 1);
    Scrutari.Ajax.check(requestParameters, "limit", apiConfig.getPlageLength() * 2);
    $.ajax({
        url: apiConfig.getJsonUrl(),
        dataType: "jsonp",
        data: requestParameters,
        success: function (data, textStatus) {
            Scrutari.Ajax.success(data, "ficheSearchResult", existingFicheSearchResultCallback);
        }
    });
};
Scrutari.Ajax.loadGeoJson = function (geojsonCallback, apiConfig, requestParameters, apiErrorCallback) {
    if (!requestParameters) {
        requestParameters = new Object();
    }
    requestParameters.type = "geojson";
    Scrutari.Ajax.check(requestParameters, "langui", apiConfig.getLangUi());
    Scrutari.Ajax.check(requestParameters, "fichefields", "titre");
    Scrutari.Ajax.check(requestParameters, "motclefields", "");
    Scrutari.Ajax.check(requestParameters, "origin", apiConfig.getOrigin());
    Scrutari.Ajax.check(requestParameters, "warnings", 1);
    Scrutari.Ajax.check(requestParameters, "version", 1);
    $.ajax({
        url: apiConfig.getJsonUrl(),
        dataType: "jsonp",
        data: requestParameters,
        success: function (data, textStatus) {
            if (data.hasOwnProperty("error")) {
                if (apiErrorCallback) {
                    apiErrorCallback(data.error);
                } else {
                    Scrutari.logError(data.error);
                }
            } else {
                Scrutari.Ajax.logWarnings(data);
                geojsonCallback(data);
            }
        }
    });
};
Scrutari.Ajax.success = function(ajaxResult, objectName, objectCallback, apiErrorCallback) {
    if (ajaxResult.hasOwnProperty("error")) {
        if (apiErrorCallback) {
            apiErrorCallback(ajaxResult.error);
        } else {
            Scrutari.logError(ajaxResult.error);
        }
    } else {
        Scrutari.Ajax.logWarnings(ajaxResult);
        if (!ajaxResult.hasOwnProperty(objectName)) {
            $.error(objectName + " object is missing in json response");
        } else {
            objectCallback(ajaxResult[objectName]);
        }
    }
};
Scrutari.Ajax.logWarnings = function (ajaxResult) {
    if (ajaxResult.hasOwnProperty("warnings")) {
        warningsMessage = "Scrutari Request Warnings [";
        for(var i = 0; i < ajaxResult.warnings.length; i++) {
            if (i > 0) {
                warningsMessage += ";";
            }
            var warning = ajaxResult.warnings[i];
            warningsMessage += "key = ";
            warningsMessage += warning.key;
            warningsMessage += " | parameter = ";
            warningsMessage += warning.parameter;
            if (warning.hasOwnProperty("value")) {
                warningsMessage += " | value = ";
                warningsMessage += warning.value;
            }
        }
        warningsMessage += "]";
        Scrutari.log(warningsMessage);
    }
};
Scrutari.Ajax.check = function (obj, name, defaultValue) {
    if (!obj.hasOwnProperty(name)) {
        obj[name] = defaultValue;
    }
};
Scrutari.Meta = function (engineInfo) {
    this.engineInfo = engineInfo;
};
Scrutari.Meta.load = function(apiConfig, callback) {
    var buffer = new Object();
    var _ajaxEnd = function() {
        var apiMeta = new Scrutari.Meta(buffer.engineInfo);
        if (callback) {
            callback(apiMeta);
        }
    };
    var _engineInfoCallback = function (engineInfo) {
        buffer.engineInfo = engineInfo;
        _ajaxEnd();
    };
    Scrutari.Ajax.loadEngineInfo(_engineInfoCallback, apiConfig);
};
Scrutari.Meta.prototype.getEngineInfo = function () {
    return this.engineInfo;
};
Scrutari.Meta.prototype.getTitle = function () {
    return this.engineInfo.metadata.title;
};
Scrutari.Meta.prototype.getDefaultOptions = function () {
    var options = {};
    var attrMap = this.engineInfo.metadata.attrMap;
    for(var key in attrMap) {
        if (key.indexOf("scrutarijs:") === 0) {
            var name = key.substring("scrutarijs:".length);
            var values = attrMap[key];
            if (values.length == 1) {
                var value = values[0];
                if (value === "false") {
                    value = false;
                } else if (value === "true") {
                    value = true;
                }
                options[name] = value;
            } else {
                options[name] = values;
            }
        }
        this[key] = options[key];
    }
    return options;
};
Scrutari.Meta.prototype.getGlobalFicheCount = function () {
    return this.engineInfo.stats.fiche;
};
Scrutari.Meta.prototype.getGlobalLangFicheCount = function (langArray) {
    var ficheCount = 0;
    var length = this.engineInfo.stats.langArray.length;
    for(var i = 0; i < length; i++) {
        var langObj = this.engineInfo.stats.langArray[i];
        if ($.inArray(langObj.lang, langArray) !== -1) {
            ficheCount += langObj.fiche;
        }
    }
    return ficheCount;
};
Scrutari.Meta.prototype.getCorpusFicheCount = function (code) {
    var corpus = this.getCorpus(code);
    if (!corpus) {
        return 0;
    }
    return corpus.stats.fiche;
};
Scrutari.Meta.prototype.getCorpusLangFicheCount = function (code, langArray) {
    var corpus = this.getCorpus(code);
    if (!corpus) {
        return 0;
    }
    var ficheCount = 0;
    var length = corpus.stats.langArray.length;
    for(var i = 0; i < length; i++) {
        var langObj = corpus.stats.langArray[i];
        if ($.inArray(langObj.lang, langArray) !== -1) {
            ficheCount += langObj.fiche;
        }
    }
    return ficheCount;
};
Scrutari.Meta.prototype.getBaseArray = function (sortFunction) {
    var array = new Array();
    var baseMap = this.engineInfo.baseMap;
    var p=0;
    for(var prop in baseMap) {
        array[p] = baseMap[prop];
        p++;
    }
    if (sortFunction) {
        array = array.sort(sortFunction);
    }
    return array;
};
Scrutari.Meta.prototype.getLangArray = function (sortFunction) {
    var array = new Array();
    var length = this.engineInfo.stats.langArray.length;
    for(var i = 0; i < length; i++) {
        array[i] = this.engineInfo.stats.langArray[i];
    }
    if (sortFunction) {
        array = array.sort(sortFunction);
    }
    return array;
};
Scrutari.Meta.prototype.getCategoryArray = function (sortFunction) {
    var array = new Array();
    if (!this.engineInfo.hasOwnProperty("categoryMap")) {
        return array;
    }
    var categoryMap = this.engineInfo.categoryMap;
    var p=0;
    for(var prop in categoryMap) {
        array[p] = categoryMap[prop];
        p++;
    }
    if (sortFunction) {
        array = array.sort(sortFunction);
    }
    return array;
};
Scrutari.Meta.prototype.getCorpusArray = function (sortFunction) {
    var array = new Array();
    var corpusMap = this.engineInfo.corpusMap;
    var p=0;
    for(var prop in corpusMap) {
        array[p] = corpusMap[prop];
        p++;
    }
    if (sortFunction) {
        array = array.sort(sortFunction);
    }
    return array;
};
Scrutari.Meta.prototype.getAttributeArray = function (type) {
    if (!this.engineInfo.hasOwnProperty("attributes")) {
        return new Array();
    }
    if (!this.engineInfo.attributes.hasOwnProperty(type)) {
        return new Array();
    }
    return this.engineInfo.attributes[type];
};
Scrutari.Meta.prototype.getBase = function (code) {
    var key = "code_" + code;
    if (this.engineInfo.baseMap.hasOwnProperty(key)) {
        return this.engineInfo.baseMap[key];
    } else {
        return null;
    }
};
Scrutari.Meta.prototype.getCorpus = function (code) {
    var key = "code_" + code;
    if (this.engineInfo.corpusMap.hasOwnProperty(key)) {
        return this.engineInfo.corpusMap[key];
    } else {
        return null;
    }
};
Scrutari.Meta.prototype.getThesaurus = function (code) {
    var key = "code_" + code;
    if (this.engineInfo.thesaurusMap.hasOwnProperty(key)) {
        return this.engineInfo.thesaurusMap[key];
    } else {
        return null;
    }
};
Scrutari.Meta.prototype.getLangLabel = function (iso) {
    if (this.engineInfo.langMap.hasOwnProperty(iso)) {
        return this.engineInfo.langMap[iso];
    } else {
        return iso;
    }
};
Scrutari.Meta.prototype.withCategory = function () {
    return this.engineInfo.hasOwnProperty("categoryMap");
};
Scrutari.Meta.prototype.getCorpusArrayForCategories = function (categoryArray) {
    var result = new Array();
    if (!this.engineInfo.hasOwnProperty("categoryMap")) {
        return result;
    }
    for(var i = 0; i < categoryArray.length; i++) {
        var categoryName = categoryArray[i];
        if (this.engineInfo.categoryMap.hasOwnProperty(categoryName)) {
            result = result.concat(this.engineInfo.categoryMap[categoryName].codecorpusArray);
        }
    }
    return result;
};
Scrutari.Meta.prototype.getCorpusArrayForBases = function (baseArray) {
    var result = new Array();
    for(var i = 0; i < baseArray.length; i++) {
        var key = "code_" + baseArray[i];
        if (this.engineInfo.baseMap.hasOwnProperty(key)) {
            result = result.concat(this.engineInfo.baseMap[key].codecorpusArray);
        }
    }
    return result;
};
Scrutari.Meta.prototype.getComplementIntitule = function(code, complementNumber) {
    var corpus = this.getCorpus(code);
    if (!corpus) {
        return "";
    }
    var key = "complement_" + complementNumber;
    if (corpus.intitules.hasOwnProperty(key)) {
        return corpus.intitules[key];
    } else {
        return key;
    }
};
Scrutari.Filter = function () {
    this.withBase = false;
    this.withCategory = false;
    this.withBase = false;
    this.withCorpus = false;
    this.baseArray = new Array();
    this.langArray = new Array();
    this.categoryArray = new Array();
    this.corpusArray = new Array();
};
Scrutari.Filter.prototype.clearArrays = function () {
    while(this.baseArray.length > 0) {
        this.baseArray.pop();
    }
    while(this.langArray.length > 0) {
        this.langArray.pop();
    }
    while(this.categoryArray.length > 0) {
        this.categoryArray.pop();
    }
    while(this.corpusArray.length > 0) {
        this.corpusArray.pop();
    }
};
Scrutari.Filter.prototype.setWithBase = function (bool) {
    this.withBase = bool;
};
Scrutari.Filter.prototype.setWithLang = function (bool) {
    this.withLang = bool;
};
Scrutari.Filter.prototype.setWithCategory = function (bool) {
    this.withCategory = bool;
};
Scrutari.Filter.prototype.setWithCorpus = function (bool) {
    this.withCorpus = bool;
};
Scrutari.Filter.prototype.addBase = function (base) {
    this.baseArray.push(base);
};
Scrutari.Filter.prototype.addLang = function (lang) {
    this.langArray.push(lang);
};
Scrutari.Filter.prototype.addCategory = function (category) {
    this.categoryArray.push(category);
};
Scrutari.Filter.prototype.addCorpus = function (corpus) {
    this.corpusArray.push(corpus);
};
Scrutari.Filter.prototype.checkRequestParameters = function (apiMeta, requestParameters) {
    if (this.hasLang()) {
        requestParameters.langlist = this.langArray.join(",");
    }
    if (this.hasBase()) {
        requestParameters.baselist = this.baseArray.join(",");
    }
    if (this.hasCategory()) {
        requestParameters.categorylist = this.categoryArray.join(",");
    }
    if (this.hasCorpus()) {
        requestParameters.corpuslist = this.corpusArray.join(",");
    }
};
Scrutari.Filter.prototype.getFilterFicheCount = function (apiMeta) {
    if (!this.hasFilter()) {
        return apiMeta.getGlobalFicheCount();
    }
    var filterFicheCount = 0;
    if (this.hasCorpusFilter()) {
        var corpusMap = this.checkCorpusMap(apiMeta);
        var completeValue = corpusMap.completeValue;
        for(var key in corpusMap) {
            if (key === 'completeValue') {
                continue;
            }
            if (corpusMap[key] === completeValue) {
                var codecorpus = parseInt(key.substring(5));
                if (this.hasLang()) {
                    filterFicheCount += apiMeta.getCorpusLangFicheCount(codecorpus, this.langArray);
                } else {
                    filterFicheCount += apiMeta.getCorpusFicheCount(codecorpus);
                }
            }
        }
    } else {
        filterFicheCount += apiMeta.getGlobalLangFicheCount(this.langArray);
    }
    return filterFicheCount;
};
Scrutari.Filter.prototype.hasFilter = function () {
    if (this.hasLang()) {
        return true;
    }
    if (this.hasBase()) {
        return true;
    }
    if (this.hasCategory()) {
        return true;
    }
    if (this.hasCorpus()) {
        return true;
    }
    return false;
};
Scrutari.Filter.prototype.hasCorpusFilter = function () {
    if (this.hasBase()) {
        return true;
    }
    if (this.hasCategory()) {
        return true;
    }
    if (this.hasCorpus()) {
        return true;
    }
    return false;
};
Scrutari.Filter.prototype.checkCorpusMap = function (apiMeta) {
    var corpusMap = new Object();
    var finalCount = 0;
    if (this.hasCategory())  {
        finalCount++;
        var arrayForCategories = apiMeta.getCorpusArrayForCategories(this.categoryArray);
        for(var i = 0; i < arrayForCategories.length; i++) {
            var key = "code_" + arrayForCategories[i];
            if (corpusMap.hasOwnProperty(key)) {
                corpusMap[key] = corpusMap[key] + 1;
            } else {
                corpusMap[key] = 1;
            }
        }
    }
    if (this.hasBase())  {
        finalCount++;
        var arrayForBases = apiMeta.getCorpusArrayForBases(this.baseArray);
        for(var i = 0; i < arrayForBases.length; i++) {
            var key = "code_" + arrayForBases[i];
            if (corpusMap.hasOwnProperty(key)) {
                corpusMap[key] = corpusMap[key] + 1;
            } else {
                corpusMap[key] = 1;
            }
        }
    }
    if (this.hasCorpus())  {
        finalCount++;
        for(var i = 0; i < this.corpusArray.length; i++) {
            var key = "code_" + this.corpusArray[i];
            if (corpusMap.hasOwnProperty(key)) {
                corpusMap[key] = corpusMap[key] + 1;
            } else {
                corpusMap[key] = 1;
            }
        }
    }
    corpusMap.completeValue = finalCount;
    return corpusMap;
};
Scrutari.Filter.prototype.hasLang = function () {
    return ((this.withLang) && (this.langArray.length > 0));
};
Scrutari.Filter.prototype.hasBase = function () {
    return ((this.withBase) && (this.baseArray.length > 0));
};
Scrutari.Filter.prototype.hasCategory = function () {
    return ((this.withCategory) && (this.categoryArray.length > 0));
};
Scrutari.Filter.prototype.hasCorpus = function () {
    return ((this.withCorpus) && (this.corpusArray.length > 0));
};
Scrutari.Result = function (ficheSearchResult, groupSortFunction) {
    this.ficheSearchResult = ficheSearchResult;
    this.ficheGroupArray = ficheSearchResult.ficheGroupArray;
    if ((groupSortFunction) && (this.ficheGroupArray.length > 1)) {
        this.ficheGroupArray = this.ficheGroupArray.sort(groupSortFunction);
    }
    this.motcleMap = new Object();
    if (ficheSearchResult.hasOwnProperty("motcleArray")) {
        var length = ficheSearchResult.motcleArray.length;
        for(var i = 0; i < length; i++) {
            var motcle = ficheSearchResult.motcleArray[i];
            this.motcleMap["code_" + motcle.codemotcle] = motcle;
        }
    }
};
Scrutari.Result.newSearch = function (apiConfig, requestParameters, callback, apiErrorCallback) {
    var _ficheSearchResultCallback = function (ficheSearchResult) {
        callback(new Scrutari.Result(ficheSearchResult, apiConfig.getGroupSortFunction()));
    };
    Scrutari.Ajax.loadFicheSearchResult(_ficheSearchResultCallback, apiConfig, requestParameters, apiErrorCallback);
};
Scrutari.Result.prototype.getQId = function () {
    return this.ficheSearchResult.qId;
};
Scrutari.Result.prototype.getQ = function () {
    return this.ficheSearchResult.q;
};
Scrutari.Result.prototype.getFicheCount = function () {
    return this.ficheSearchResult.ficheCount;
};
Scrutari.Result.prototype.getFicheGroupType = function () {
    return this.ficheSearchResult.ficheGroupType;
};
Scrutari.Result.prototype.getNoneFicheArray = function () {
    if (this.ficheGroupArray.length === 0) {
        return new Array();
    }
    return this.ficheGroupArray[0].ficheArray;
};
Scrutari.Result.prototype.selectNoneFicheArray = function (plageLength, plageNumber) {
    var selectionArray = new Array();
    if (this.ficheGroupArray.length === 0) {
        return selectionArray;
    }
    var ficheArray = this.ficheGroupArray[0].ficheArray;
    var startIndex = plageLength * (plageNumber - 1);
    var length = ficheArray.length;
    if (startIndex >= length) {
        return selectionArray;
    }
    var min = Math.min(ficheArray.length, startIndex + plageLength);
    for(var i = startIndex; i < min; i++) {
        selectionArray.push(ficheArray[i]);
    }
    return selectionArray;
};
Scrutari.Result.prototype.isNonePlageLoaded = function (plageLength, plageNumber) {
    if (this.ficheGroupArray.length === 0) {
        return true;
    }
    var ficheCount = this.ficheSearchResult.ficheCount;
    var ficheArray = this.ficheGroupArray[0].ficheArray;
    var length = ficheArray.length;
    if (length === ficheCount) {
        return true;
    }
    var endIndex = (plageLength * plageNumber) - 1;
    if (endIndex < length) {
        return true;
    }
    return false;
};
Scrutari.Result.prototype.loadNonePlage = function (apiConfig, plageLength, plageNumber, callback) {
    if (this.ficheGroupArray.length === 0) {
        return true;
    }
    var group = this.ficheGroupArray[0];
    if (!group) {
        return;
    }
    var ficheCount = this.ficheSearchResult.ficheCount;
    var ficheArray = group.ficheArray;
    var length = ficheArray.length;
    if (length === ficheCount) {
        return;
    }
    var _existingFicheSearchResultCallback = function (ficheSearchResult) {
        var newCount = ficheSearchResult.ficheGroupArray.length;
        if (newCount > 0) {
            group.ficheArray = group.ficheArray.concat(ficheSearchResult.ficheGroupArray[0].ficheArray);
        }
        if (callback) {
            callback();
        }
    };
    var requestParameters = {
        qid: this.ficheSearchResult.qId,
        start: length +1,
        limit: (plageLength * (plageNumber + 2)) - length
    };
    Scrutari.Ajax.loadExistingFicheSearchResult(_existingFicheSearchResultCallback, apiConfig, requestParameters); 
};
Scrutari.Result.prototype.isCategoryPlageLoaded = function (categoryName, plageLength, plageNumber) {
    var group = this.getFicheGroupByCategoryName(categoryName);
    if (!group) {
        return true;
    }
    var categoryFicheCount = group.ficheCount;
    var ficheArray = group.ficheArray;
    var length = ficheArray.length;
    if (length === categoryFicheCount) {
        return true;
    }
    var endIndex = (plageLength * plageNumber) - 1;
    if (endIndex < length) {
        return true;
    }
    return false;
};
Scrutari.Result.prototype.loadCategoryPlage = function (apiConfig, categoryName, plageLength, plageNumber, callback) {
    var group = this.getFicheGroupByCategoryName(categoryName);
    if (!group) {
        return;
    }
    var categoryFicheCount = group.ficheCount;
    var ficheArray = group.ficheArray;
    var length = ficheArray.length;
    if (length === categoryFicheCount) {
        return;
    }
    var _existingFicheSearchResultCallback = function (ficheSearchResult) {
        var newCount = ficheSearchResult.ficheGroupArray.length;
        for(var i = 0; i < newCount; i++) {
            var newGroup = ficheSearchResult.ficheGroupArray[i];
            if (newGroup.category.name === group.category.name) {
                group.ficheArray = group.ficheArray.concat(newGroup.ficheArray);
            }
        }
        callback();
    };
    var requestParameters = {
        qid: this.ficheSearchResult.qId,
        start: length +1,
        limit: (plageLength * (plageNumber + 2)) - length,
        starttype: "in:" + categoryName
    };
    Scrutari.Ajax.loadExistingFicheSearchResult(_existingFicheSearchResultCallback, apiConfig, requestParameters); 
};
Scrutari.Result.prototype.selectCategoryFicheArray = function (categoryName, plageLength, plageNumber) {
    var selectionArray = new Array();
    var ficheArray = this.getCategoryFicheArrayByName(categoryName);
    var startIndex = plageLength * (plageNumber - 1);
    var length = ficheArray.length;
    if (startIndex >= length) {
        return selectionArray;
    }
    var min = Math.min(ficheArray.length, startIndex + plageLength);
    for(var i = startIndex; i < min; i++) {
        selectionArray.push(ficheArray[i]);
    }
    return selectionArray;
};
Scrutari.Result.prototype.getCategoryCount = function () {
    return this.ficheGroupArray.length;
};
Scrutari.Result.prototype.getCategory = function (index) {
    return this.ficheGroupArray[index].category;
};
Scrutari.Result.prototype.getFicheGroupByCategoryName = function (categoryName) {
    var groupCount = this.ficheGroupArray.length;
    for(var i = 0; i < groupCount; i++) {
        var group = this.ficheGroupArray[i];
        if ((group.category) && (group.category.name === categoryName)) {
            return group;
        }
    }
    return null;
};
Scrutari.Result.prototype.getCategoryFicheCount = function (index) {
    return this.ficheGroupArray[index].ficheCount;
};
Scrutari.Result.prototype.getCategoryFicheCountbyName = function (categoryName) {
    var groupCount = this.ficheGroupArray.length;
    for(var i = 0; i < groupCount; i++) {
        var group = this.ficheGroupArray[i];
        if ((group.category) && (group.category.name === categoryName)) {
            return group.ficheCount;
        }
    }
    return 0;
};
Scrutari.Result.prototype.getCategoryFicheArray = function (index) {
    return this.ficheGroupArray[index].ficheArray;
};
Scrutari.Result.prototype.getCategoryFicheArrayByName = function (categoryName) {
    var categoryCount = this.getCategoryCount();
    for(var i = 0; i < categoryCount; i++) {
        var category = this.getCategory(i);
        if (category.name === categoryName) {
            return this.getCategoryFicheArray(i);
        }
    }
    return new Array();
};
Scrutari.Result.prototype.getMotcle = function (code) {
    var key = "code_" + code;
    if (this.motcleMap.hasOwnProperty(key)) {
        return this.motcleMap[key];
    } else {
        return null;
    }
};
Scrutari.Utils = {};
Scrutari.Utils.divideIntoColumns = function (objectArray, jqArgument, objectTemplate) {
    var objectCount = objectArray.length;
    if (objectCount === 0) {
        return;
    }
    var $elements = Scrutari.convert(jqArgument);
    var elementCount = $elements.length;
    if (elementCount === 0) {
        Scrutari.log("HtmlElement selection with jqArgument is empty ");
        return;
    }
    var objectCount = objectArray.length;
    if (objectCount <= elementCount) {
        for(var i = 0; i < objectCount; i++) {
            $($elements[i]).append(objectTemplate(objectArray[i]));
        }
        return;
    }
    var modulo = objectCount % elementCount;
    var columnLength = (objectCount - modulo) / elementCount;
    var start = 0;
    var stop = 0;
    for(var i = 0; i< elementCount; i++) {
        var $element = $($elements[i]);
        stop += columnLength;
        if (i < modulo) {
            stop++;
        }
        for(var j = start; j < stop; j++) {
            $element.append(objectTemplate(objectArray[j]));
        }
        start = stop;
    }
};
Scrutari.Utils.checkPagination = function (ficheCount, plageLength, currentPlageNumber, jqArgument, paginationTemplate) {
    var $elements = Scrutari.convert(jqArgument);
    $elements.empty();
    var plageCount;
    if (ficheCount <= plageLength) {
        plageCount = 1;
        return;
    } else {
        var modulo = ficheCount % plageLength;
        plageCount = (ficheCount - modulo) / plageLength;
        if (modulo > 0) {
            plageCount ++;
        }
    }
    if (currentPlageNumber > plageCount) {
        currentPlageNumber = plageCount;
    }
    var plageNumberStart = 1;
    var plageNumberEnd = 9;
    if (currentPlageNumber > 6) {
        plageNumberStart = currentPlageNumber - 3;
        plageNumberEnd = currentPlageNumber + 3;
    }
    if (plageNumberEnd > plageCount) {
        plageNumberEnd = plageCount;
    }
    if (plageNumberStart > 1) {
        $elements.append(paginationTemplate({
                number: 1,
                title: "1",
                state: 'enabled'
        }));
        $elements.append(paginationTemplate({
                number: 0,
                title: "…",
                state: 'disabled'
        }));
    }
    for(var i = plageNumberStart; i <= plageNumberEnd; i++) {
        var state = 'enabled';
        if (i == currentPlageNumber) {
            state = 'active';
        }
        var html = paginationTemplate({
                number: i,
                title: i.toString(),
                state: state
        });
        $elements.append(html);
    }
    if (plageNumberEnd < plageCount) {
        $elements.append(paginationTemplate({
                number: 0,
                title: "…",
                state: 'disabled'
        }));
    }
};
Scrutari.Utils.disable = function (jqArgument) {
    var $elements = Scrutari.convert(jqArgument);
    $elements.prop('disabled', true);
    return $elements;
};
Scrutari.Utils.enable = function (jqArgument) {
    var $elements = Scrutari.convert(jqArgument);
    $elements.prop('disabled', false);
    return $elements;
};
Scrutari.Utils.uncheck = function (jqArgument) {
    var $elements = Scrutari.convert(jqArgument);
    $elements.prop('checked', false);
    return $elements;
};
Scrutari.Utils.check = function (jqArgument) {
    var $elements = Scrutari.convert(jqArgument);
    $elements.prop('checked', true);
    return $elements;
};
Scrutari.Utils.toggle = function (jqElement, stateDataKey) {
    var state = jqElement.data(stateDataKey);
    if (state === 'off') {
        state = 'on';
    } else {
        state = 'off';
    }
    jqElement.data(stateDataKey, state);
    return state;
};
Scrutari.Utils.toggle.getState = function (jqArgument, stateDataKey) {
    var $elements = Scrutari.convert(jqArgument);
    return $elements.data(stateDataKey);
};
Scrutari.Utils.toggle.disabled = function (jqArgument, state) {
    var $elements = Scrutari.convert(jqArgument);
    if (state === 'off') {
        $elements.prop('disabled', true);
    } else {
        $elements.prop('disabled', false);
    }
    return $elements;
};
Scrutari.Utils.toggle.text = function (jqArgument, alterDataKey) {
    var $elements = Scrutari.convert(jqArgument);
    var length = $elements.length;
    for(var i = 0; i < length; i++) {
        var jqEl = $($elements[i]);
        var currentText =jqEl.text();
        var alterText = jqEl.data(alterDataKey);
        jqEl.text(alterText);
        jqEl.data(alterDataKey, currentText);
    }
    return $elements;
};
Scrutari.Utils.toggle.classes = function (jqArgument, state, onClass, offClass) {
    var $elements = Scrutari.convert(jqArgument);
    if (state === 'off') {
        $elements.addClass(offClass).removeClass(onClass);
    } else {
        $elements.removeClass(offClass).addClass(onClass);
    }
    return $elements;
};
Scrutari.Utils.sortBaseByFicheCountDesc = function (base1, base2) {
    var count1 = base1.stats.fiche;
    var count2 = base2.stats.fiche;
    if (count1 > count2) {
        return -1;
    } else if (count1 < count2) {
        return 1;
    } else {
        var code1 = base1.codebase;
        var code2 = base2.codebase;
        if (code1 < code2) {
            return -1;
        } else if (code1 > code2) {
            return 1;
        } else {
            return 0;
        }
    }
};
Scrutari.Utils.sortLangByFicheCountDesc = function (lang1, lang2) {
    var count1 = lang1.fiche;
    var count2 = lang2.fiche;
    if (count1 > count2) {
        return -1;
    } else if (count1 < count2) {
        return 1;
    } else {
        var code1 = lang1.lang;
        var code2 = lang2.lang;
        if (code1 < code2) {
            return -1;
        } else if (code1 > code2) {
            return 1;
        } else {
            return 0;
        }
    }
};
Scrutari.Utils.sortCategoryByRankDesc = function (category1, category2) {
    var count1 = category1.rank;
    var count2 = category2.rank;
    if (count1 > count2) {
        return -1;
    } else if (count1 < count2) {
        return 1;
    } else {
        var code1 = category1.name;
        var code2 = category2.name;
        if (code1 < code2) {
            return -1;
        } else if (code1 > code2) {
            return 1;
        } else {
            return 0;
        }
    }
};
Scrutari.Utils.sortCorpusByFicheCountDesc = function (corpus1, corpus2) {
    var count1 = corpus1.stats.fiche;
    var count2 = corpus2.stats.fiche;
    if (count1 > count2) {
        return -1;
    } else if (count1 < count2) {
        return 1;
    } else {
        var code1 = corpus1.codecorpus;
        var code2 = corpus2.codecorpus;
        if (code1 < code2) {
            return -1;
        } else if (code1 > code2) {
            return 1;
        } else {
            return 0;
        }
    }
};
Scrutari.Utils.mark = function (markArray, classAttribute) {
    if (!classAttribute) {
        classAttribute = "scrutari-Mark";
    }
    var html = "";
    var length = markArray.length;
    for (var i = 0; i < length; i++) {
        var obj = markArray[i];
        if (typeof obj === 'string') {
            html += obj;
        } else if (obj.s) {
            html += "<span class='" + classAttribute + "'>";
            html += Scrutari.escape(obj.s);
            html += "</span>";
        }
    }
    return html;
};
Scrutari.Utils.buildPaginationTemplate = function (paginationIdPrefix) {
    var _paginationTemplate = function (obj) {
        var html = "";
        if (obj.state === 'active') {
            html += "<li class='active'>";
            html += "<span>";
            html += Scrutari.escape(obj.title);
            html += "</span>";
            html += "</li>";
        } else if (obj.state === 'disabled') {
            html += "<li class='disabled'>";
            html += "<span>";
            html += Scrutari.escape(obj.title);
            html += "</span>";
            html += "</li>";
        } else {
            html += "<li>";
            html += "<a href='#" + paginationIdPrefix + obj.number + "'>";
            html += Scrutari.escape(obj.title);
            html += "</a>";
            html += "</li>";
        }
        return html;   
    };
    return _paginationTemplate;
};
Scrutari.Utils.buildFicheTemplate = function (client, apiResult, target) {
    var apiMeta = client._apiMeta;
     var _attrToString = function (mattrMap, attributeArray) {
         var html = "";
         var length = attributeArray.length;
         for(var i = 0; i < length; i++) {
            var attribute = attributeArray[i];
            if (mattrMap.hasOwnProperty(attribute.name)) {
                 var valueArray = mattrMap[attribute.name];
                 var valLength = valueArray.length;
                 html += "<p class='scrutari-fiche-Attribute'>";
                 html += '<span class="scrutari-label-Attribute">';
                 html += attribute.title;
                 html += client.loc("_ colon");
                 html += '</span>';
                 if (attribute.type === "block") {
                     html += '</p><p class="scrutari-fiche-AttrBlocks">…<br/>';
                     for(var j = 0; j< valLength; j++) {
                        if (j > 0) {
                            html += "<br/>…<br/>";
                        }
                        html +=Scrutari.Utils.mark(valueArray[j]);
                     }
                     html += '<br/>…</p>';
                 } else {
                     html += ' ';
                     for(var j = 0; j< valLength; j++) {
                        if (j > 0) {
                            html += ", ";
                        }
                        html +=Scrutari.Utils.mark(valueArray[j]);
                     }
                     html += "</p>";
                 }
            }
         }
         return html;
     };
     var _ficheTemplate = function (fiche) {
        var html = "";
        html += "<div class='scrutari-fiche-Block' id='ficheBlock_" + fiche.codefiche + "'>";
        if (fiche.hasOwnProperty("icon")) {
            html += "<div class='scrutari-fiche-Icon'><img alt='' src='" + fiche.icon + "'/></div>";
        }
        html += "<p class='scrutari-fiche-Titre'>";
        html += "<a href='" + fiche.href + "'";
        html += " class='scrutari-fiche-Link'";
        html += " id='ficheLink_" + fiche.codefiche + "'";
        if (target.length > 0) {
            html += " target='" + target + "'";
        }
        html += ">";
        if (fiche.hasOwnProperty("mtitre")) {
            html +=Scrutari.Utils.mark(fiche.mtitre);
        } else {
            html += fiche.href;
        }
        html += "</a>";
        html += "</p>";
        if (fiche.hasOwnProperty("msoustitre")) {
            html += "<p class='scrutari-fiche-Soustitre'>";
            html +=Scrutari.Utils.mark(fiche.msoustitre);
            html += "</p>";
        }
        if (fiche.hasOwnProperty("annee")) {
            html += "<p class='scrutari-fiche-Annee'>";
            html += fiche.annee;
            html += "</p>";
        }
        if (fiche.hasOwnProperty("mattrMap")) {
            html += _attrToString(fiche.mattrMap, apiMeta.getAttributeArray("primary"));
        }
        if (fiche.hasOwnProperty("mcomplementArray")) {
            var complementLength = fiche.mcomplementArray.length;
            for(var i = 0; i < complementLength; i++) {
                var complement = fiche.mcomplementArray[i];
                html += "<p class='scrutari-fiche-Complement'>";
                html += '<span class="scrutari-label-Complement">';
                html += apiMeta.getComplementIntitule(fiche.codecorpus, complement.num);
                html += client.loc("_ colon");
                html += '</span> ';
                html +=Scrutari.Utils.mark(complement.mcomp);
                html += "</p>";
            }
        }
        if (fiche.hasOwnProperty("mattrMap")) {
            html += _attrToString(fiche.mattrMap, apiMeta.getAttributeArray("secondary"));
        }
        if (fiche.hasOwnProperty('codemotcleArray')) {
            var length = fiche.codemotcleArray.length;
            html += '<p class="scrutari-fiche-Motcle">';
            html += '<span class="scrutari-label-Motcle">';
            if (length === 1) {
                html += client.loc("_ fiche_motscles_one");
            } else {
                html += client.loc("_ fiche_motscles_many");
            }
            html += '</span> ';
            for(var i = 0; i < length; i++) {
                if (i > 0) {
                    html += ", ";
                }
                var code = fiche.codemotcleArray[i];
                var motcle = apiResult.getMotcle(code);
                var libelleLength = motcle.mlibelleArray.length;
                for(var j = 0; j < libelleLength; j++) {
                    if (j > 0) {
                        html += "/";
                    }
                    html +=Scrutari.Utils.mark(motcle.mlibelleArray[j].mlib);
                }
            }
            html += "</p>";
        }
        html += '</div>';
        return html;
    };
    return _ficheTemplate;
};
Scrutari.Utils.formatSearchSequence = function (client, apiResult) {
    var q = apiResult.getQ();
    q = q.replace(/\&\&/g, client.loc('_ and'));
    q = q.replace(/\|\|/g, client.loc('_ or'));
    return q;
};
Scrutari.Loc = function (map) {
    if (map) {
        this.map = map;
    } else {
        this.map = new Object();
    }
};
Scrutari.Loc.prototype.putAll = function (map) {
    for(var key in map) {
        this.map[key] = map[key];
    }
};
Scrutari.Loc.prototype.putLoc = function (locKey, locText) {
    this.map[locKey] = locText;
};
Scrutari.Loc.prototype.loc = function (locKey) {
    if (!this.map.hasOwnProperty(locKey)) {
        return locKey;
    }
    var text = this.map[locKey];
    var argLength = arguments.length;
    if (argLength > 1) {
        for(var i = 0; i < argLength; i++) {
            var p = i -1;
            var mark = "{" + p + "}";
            text = text.replace(mark, arguments[i]);
        }
    }
    return text;
};
Scrutari.Loc.prototype.escape = function (locKey) {
    return Scrutari.escape(this.loc.apply(this, arguments));
};
Scrutari.Client = function (apiConfig) {
    this._apiConfig = apiConfig;
    this._locInstance = new Scrutari.Loc();
    this._apiMeta = null;
    this._isWaiting = false;
    this._currentApiResult = null;
    this._historyNumber = 0;
    this.withCorpus = false;
    this.baseSort = "fiche-count";
    this.corpusSort = "fiche-count";
    this.mode = "";
    this.target = "_blank";
    this.initialQuery = "";
    this.initialQId = "";
    this.bootstrapVersion = "3";
    this.permalinkPattern = null;
    this.changePlageCallback = null;
    this.hiddenList = null;
    this.templateFactory = null;
};
Scrutari.Client.prototype.loc = function (locKey) {
    return this._locInstance.loc(locKey);
};
Scrutari.Client.prototype.toPermalink = function (qId) {
    if (this.permalinkPattern) {
        return this._apiConfig.getPermalinkUrl(qId, this.permalinkPattern);
    }
    return null;
};
Scrutari.Client.prototype.newHistoryNumber = function () {
    this._historyNumber++;
    return this._historyNumber;
};
Scrutari.Client.prototype.isHidden = function (elementName) {
    return $.inArray(elementName, this.hiddenList) > -1;
};
Scrutari.Client.prototype.getTemplate = function (templateName, templateParameter) {
    if (this.templateFactory) {
        if ($.isFunction(this.templateFactory)) {
            var template = this.templateFactory(this, templateName, templateParameter);
            if (template) {
                return template;
            }
        } else if ($.isArray(this.templateFactory)) {
            var factoryLength = this.templateFactory.length;
            if (factoryLength > 0) {
                for(var i = (factoryLength -1); i >= 0; i--) {
                    var templateFactory = this.templateFactory[i];
                    var template = templateFactory(this, templateName, templateParameter);
                    if (template) {
                        return template;
                    }
                }
            }
        }
    }
    switch(templateName) {
        case "base":
            return Scrutari.Client.baseTemplate;
        case "category":
            return Scrutari.Client.categoryTemplate;
        case "corpus":
            return Scrutari.Client.corpusTemplate;
        case "lang":
            var apiMeta = this._apiMeta;
            return function (lang) {
                var code = lang.lang;
                var html = "";
                html += "<p>";
                html += "<label class='scrutari-Label'><input type='checkbox' name='lang' value='" + code + "'> ";
                var label = apiMeta.getLangLabel(code);
                if (label !== code) {
                    html += label;
                    html += " ";
                }
                html += "[" + code + "]";
                html += " (" + lang.fiche +")";
                html += "</label>";
                html += "</p>";
                return html;
            };
        case "pagination":
            return Scrutari.Utils.buildPaginationTemplate(templateParameter);
        case "fiche":
            return Scrutari.Utils.buildFicheTemplate(this, templateParameter, this.target);
        default:
            return function () {
                return "Unknwon template: " + templateName;
            };
    }
};
Scrutari.Client.prototype.checkOptions = function (options, apiMeta) {
    var defaultOptions = apiMeta.getDefaultOptions();
    for(var key in defaultOptions) {
        if (key.substring(0,1) !== "_") {
            this[key] = defaultOptions[key];
        }
    }
    if (options) {
        for(var key in options) {
            if (key.substring(0,1) !== "_") {
                this[key] = options[key];
            }
        }
        if (options.locMap) {
            this._locInstance.putAll(options.locMap);
        }
    }
    if (this.hiddenList) {
        if ($.type(this.hiddenList) === "string") {
            this.hiddenList = this.hiddenList.split(",");
        }
        for(var i = 0, len = this.hiddenList.length; i < len; i++) {
            this.hiddenList[i] = $.trim(this.hiddenList[i]);
        }
    } else {
        this.hiddenList = new Array();
    }
};
Scrutari.Client.init = function (apiConfig, options, callback) {
    Scrutari.Meta.load(apiConfig, function (apiMeta) {
        var client = new Scrutari.Client(apiConfig);
        client._apiMeta = apiMeta;
        client.checkOptions(options, apiMeta);
        Scrutari.Client.Ui.init(client);
        if (callback) {
            callback(client);
        }
    });
};
Scrutari.Client.baseTemplate = function (base) {
    var code = base.codebase;
    var html = "";
    html += "<p>";
    html += "<label class='scrutari-Label'><input type='checkbox' name='base' value='" + code + "'> ";
    if (base.baseicon) {
        html += "<img src='" + base.baseicon + "' alt='' class='scrutari-panel-Icon'> ";
    }
    html += Scrutari.escape(base.intitules.short);
    html += " (" + base.stats.fiche +")";
    html += "</label>";
    html += "</p>";
    return html;
};
Scrutari.Client.categoryTemplate = function (category) {
    var name = category.name;
    var html = "";
    html += "<p>";
    html += "<label class='scrutari-Label'><input type='checkbox' name='category' value='" + name + "'> ";
    html +=  Scrutari.escape(category.title);
    html += " (" + category.stats.fiche +")";
    html += "</label>";
    html += "</p>";
    return html;
};
Scrutari.Client.corpusTemplate = function (corpus) {
    var code = corpus.codecorpus;
    var html = "";
    html += "<p>";
    html += "<label class='scrutari-Label'><input type='checkbox' name='corpus' value='" + code + "'> ";
    html += Scrutari.escape(corpus.intitules.corpus);
    html += " (" + corpus.stats.fiche +")";
    html += "</label>";
    html += "</p>";
    return html;
};
Scrutari.Client.Result = {};
Scrutari.Client.Result.show = function (client, apiResult, searchOrigin) {
    var bootstrapV2 = ((client.bootstrapVersion) && (client.bootstrapVersion.lastIndexOf("2", 0) === 0));
    var _getPaginationHtml = function (bottomPagination, categoryName) {
        var paginationHtml = "";
        if (bootstrapV2) {
            paginationHtml += "<div class='pagination'><ul class='scrutari-result-Pagination";
            if (category) {
                paginationHtml += " scrutari-Pagination_" + categoryName;
            }
            paginationHtml +=  "'></ul></div>";
        } else {
            paginationHtml += "<ul class='pagination scrutari-result-Pagination";
            if (category) {
                paginationHtml += " scrutari-Pagination_" + categoryName;
            }
            if (bottomPagination) {
                paginationHtml += " scrutari-BottomPagination";
            }
            paginationHtml +=  "'></ul>";
        }
        return paginationHtml;
    };
    Scrutari.Client.Ui.show(".scrutari-HiddenAtStart");
    var ficheCount = apiResult.getFicheCount();
    Scrutari.Client.Ui.setCount("#resultFicheCount", ficheCount, client._apiConfig.getLangUi());
    var $ficheDisplayBlock = $("#ficheDisplayBlock");
    $ficheDisplayBlock.empty();
    if (searchOrigin === "mainsearch") {
        client._currentApiResult = apiResult;
        Scrutari.Client.Result.addToHistory(client, apiResult);
    } else if (searchOrigin === "subsearch") {
        var subsearchText =  "+ " + Scrutari.Utils.formatSearchSequence(client, apiResult) + " = " + apiResult.getFicheCount();
        $(".scrutari-history-Active").find(".scrutari-CurrentSubsearchSequence").text(subsearchText);
    }
    if (ficheCount === 0) {
        Scrutari.Client.Ui.hide(".scrutari-HiddenIfEmpty");
        $ficheDisplayBlock.html("<em>" + client.loc("_ result_none")+ "</em>");
        return;
    }
    Scrutari.Client.Ui.show(".scrutari-HiddenIfEmpty");
    var qId = apiResult.getQId();
    var permalink = client.toPermalink(qId);
    if (permalink) {
        $("#permalinkLink").attr("href", permalink);
    }
    $("#odsLink").attr("href", client._apiConfig.getOdsUrl(qId));
    $("#atomLink").attr("href", client._apiConfig.getAtomUrl(qId));
    if (apiResult.getFicheGroupType() === 'category') {
        var categoryCount = apiResult.getCategoryCount();
        var tabHtml = '<ul class="nav nav-tabs" role="tablist">';
        var contentHtml = "<div class='tab-content'>";
        for(var i = 0; i < categoryCount; i++) {
            var category = apiResult.getCategory(i);
            tabHtml += "<li";
            contentHtml += "<div class='tab-pane";
            if (i === 0) {
                tabHtml += " class='active'";
                contentHtml += " active";
            }
            tabHtml += "><a class='scrutari-Onglet' role='tab' data-toggle='tab' href='#categoryTabContent_"
                + category.name
                + "'>"
                + Scrutari.escape(category.title)
                + " ("
                + apiResult.getCategoryFicheCount(i)
                + ")"
                + "</a></li>";
            contentHtml += "' id='categoryTabContent_" + category.name + "'>";
            contentHtml += _getPaginationHtml(false, category.name);
            contentHtml += "<div id='categoryFiches_" + category.name + "'></div>";
            contentHtml += _getPaginationHtml(true, category.name);
            contentHtml += "</div>";
        }
        tabHtml += "</ul>";
        contentHtml += "</div>";
        $ficheDisplayBlock.html(tabHtml + contentHtml);
        for(var i = 0; i < categoryCount; i++) {
            var category = apiResult.getCategory(i);
            Scrutari.Client.Result.categoryChangePlage(client, category.name, apiResult, client._apiConfig.getPlageLength(), 1);
        }
    } else {
        $ficheDisplayBlock.html(_getPaginationHtml(false, null) + "<div id='noneFiches'></div>" + _getPaginationHtml(true, null));
        Scrutari.Client.Result.noneChangePlage(client, apiResult, client._apiConfig.getPlageLength(), 1);
    }
};
Scrutari.Client.Result.displayError = function (client, error, searchOrigin) {
    if (error.parameter !== "q") {
        Scrutari.logError(error);
        return;
    }
    var title =  client.loc(error.key);
    if (title !== error.key) {
        var alertMessage = title;
        if (error.hasOwnProperty("array"))  {
            alertMessage += client.loc('_ colon');
            for(var i = 0; i < error.array.length; i++) {
                alertMessage += "\n";
                var obj = error.array[i];
                alertMessage += "- ";
                alertMessage += client.loc(obj.key);
                if (obj.hasOwnProperty("value")) {
                    alertMessage += client.loc('_ colon');
                    alertMessage += " ";
                    alertMessage += obj.value;
                }
            }
        }
        alert(alertMessage);
    } else {
        Scrutari.logError(error);
    }
};
Scrutari.Client.Result.noneChangePlage = function (client, apiResult, plageLength, plageNumber) {
    var html = "";
    if (!apiResult.isNonePlageLoaded(plageLength, plageNumber)) {
        if (client._isWaiting) {
            return;
        }
        $("#noneFiches").html("<span class='scrutari-icon-Loader'></span> " + client.loc('_ loading_plage'));
        var _plageCallback = function () {
            client._isWaiting = false;
            Scrutari.Client.Result.noneChangePlage(client, apiResult, plageLength, plageNumber);
        };
        client._isWaiting = true;
        apiResult.loadNonePlage(client._apiConfig, plageLength, plageNumber, _plageCallback);
        return;
    }
    var plageFicheArray = apiResult.selectNoneFicheArray(plageLength, plageNumber);
    var ficheTemplate =  client.getTemplate("fiche", apiResult);
    for(var i = 0; i < plageFicheArray.length; i++) {
        html += ficheTemplate(plageFicheArray[i]);
    }
    $("#noneFiches").html(html);
    var paginationIdPrefix = "pagination_";
    Scrutari.Utils.checkPagination(apiResult.getFicheCount(), plageLength, plageNumber, ".scrutari-Pagination", client.getTemplate("pagination", paginationIdPrefix));
    $(".scrutari-Pagination a").click(function () {
        var idx = this.href.indexOf("#" + paginationIdPrefix);
        var newPlageNumber = parseInt(this.href.substring(idx + 1 + paginationIdPrefix.length));
        Scrutari.Client.Result.noneChangePlage(client, apiResult, plageLength, newPlageNumber);
        return false;
    });
    $(".scrutari-Pagination.scrutari-BottomPagination a").click(function () {
        Scrutari.Client.Ui.scrollToResult();
    });
    if (client.changePlageCallback) {
        client.changePlageCallback(plageFicheArray, {client: client, apiResult: apiResult,  plageLength:plageLength, plageNumber:plageNumber});
    }
};
Scrutari.Client.Result.categoryChangePlage = function (client, categoryName, apiResult, plageLength, plageNumber) {
    var html = "";
    if (!apiResult.isCategoryPlageLoaded(categoryName, plageLength, plageNumber)) {
        $("#categoryFiches_" + categoryName).html("<span class='scrutari-icon-Loader'></span> " + client.loc('_ loading_plage'));
        var _plageCallback = function () {
            client._isWaiting = false;
            Scrutari.Client.Result.categoryChangePlage(client, categoryName, apiResult, plageLength, plageNumber);
        };
        if (client._isWaiting) {
            return;
        }
        client._isWaiting = true;
        apiResult.loadCategoryPlage(client._apiConfig, categoryName, plageLength, plageNumber, _plageCallback);
        return;
    }
    var plageFicheArray = apiResult.selectCategoryFicheArray(categoryName, plageLength, plageNumber);
    var ficheTemplate =  client.getTemplate("fiche", apiResult);
    for(var i = 0; i < plageFicheArray.length; i++) {
        html += ficheTemplate(plageFicheArray[i]);
    }
    $("#categoryFiches_" + categoryName).html(html);
    var paginationIdPrefix = "pagination_" + categoryName + "_";
    Scrutari.Utils.checkPagination(apiResult.getCategoryFicheCountbyName(categoryName), plageLength, plageNumber, ".scrutari-Pagination_" + categoryName, client.getTemplate("pagination", paginationIdPrefix));
    $(".scrutari-Pagination_" + categoryName + " a").click(function () {
        var idx = this.href.indexOf("#" + paginationIdPrefix);
        var newPlageNumber = parseInt(this.href.substring(idx + 1 + paginationIdPrefix.length));
        Scrutari.Client.Result.categoryChangePlage(client, categoryName, apiResult, plageLength, newPlageNumber);
        return false;
    });
    $(".scrutari-Pagination_" + categoryName + ".scrutari-BottomPagination a").click(function () {
        Scrutari.Client.Ui.scrollToResult();
    });
    if (client.changePlageCallback) {
        client.changePlageCallback(plageFicheArray, {client: client, categoryName: categoryName, apiResult: apiResult,  plageLength:plageLength, plageNumber:plageNumber});
    }
};
Scrutari.Client.Result.addToHistory = function (client, apiResult) {
    var $resultHistoryList = $("#resultHistoryList");
    if (!Scrutari.exists($resultHistoryList)) {
        Scrutari.Client.Result.updateSubsearch(client, apiResult);
        return;
    } else {
        Scrutari.Client.Result.activateResult(client, apiResult);
    }
    var historyId = "searchHistory_" + client.newHistoryNumber();
    var withFiche = (apiResult.getFicheCount() > 0);
    var html = "";
    html += "<button type='button' class='close hidden' title='" + client.loc('_ button_remove') + "'><span aria-hidden='true'>&times;</span><span class='sr-only'>" + client.loc('_ button_remove') + "</span></button>";
    if (withFiche) {
        html += "<a href='#" + historyId + "'>";
    }
    html += Scrutari.Utils.formatSearchSequence(client, apiResult);
    html += " (" + apiResult.getFicheCount() + ")";
    if (withFiche) {
        html += "</a>";
    }
    html += "<br/><span class='scrutari-CurrentSubsearchSequence'></span>";
    var $new = $("<div/>").html(html).attr("id", historyId).addClass("scrutari-history-Block").addClass("scrutari-history-Active");
    var _close = function() {
        $("#" + historyId).remove();
    };
    var _load = function () {
        var $historyId = $("#" + historyId);
        var historyApiResult = $historyId.data("apiResult");
        client._currentApiResult = historyApiResult;
        Scrutari.Client.Result.activateResult(client, historyApiResult);
        $historyId.addClass("scrutari-history-Active");
        Scrutari.Client.Ui.hide($historyId.find("button"));
        Scrutari.Client.Result.show(client, historyApiResult);
        return false;
    };
    $new.children("button").click(_close);
    $new.children("a").click(_load);
    $new.data("apiResult", apiResult);
    $resultHistoryList.prepend($new);
};
Scrutari.Client.Result.activateResult = function (client, apiResult) {
    var $elements = $(".scrutari-history-Active");
    $elements.removeClass("scrutari-history-Active");
    $elements.find(".scrutari-CurrentSubsearchSequence").empty();
    Scrutari.Client.Ui.show($elements.find("button"));
    Scrutari.Client.Result.updateSubsearch(client, apiResult);
};
Scrutari.Client.Result.updateSubsearch = function (client, apiResult) {
    $(".scrutari-CurrentSearchSequence").text(Scrutari.Utils.formatSearchSequence(client, apiResult) + " (" + apiResult.getFicheCount() + ")");
};
Scrutari.Client.Ui = {};
Scrutari.Client.Ui.show = function (jqArgument) {
    var $elements = Scrutari.convert(jqArgument);
    return $elements.removeClass("hidden");
};
Scrutari.Client.Ui.hide = function (jqArgument) {
    var $elements = Scrutari.convert(jqArgument);
    return $elements.addClass("hidden");
};
Scrutari.Client.Ui.scrollToResult = function () {
    $(window).scrollTop($("#resultArea").offset().top);
};
Scrutari.Client.Ui.setCount = function (parentJqArgument, count, locale) {
    var $parent = Scrutari.convert(parentJqArgument);
    var $elements = $parent.find(".scrutari-stats-Count");
    var html = '';
    if (Number.prototype.toLocaleString) {
        html += count.toLocaleString(locale);
    } else {
        html += count;
    }
    $elements.html(html);
    return $elements;
};
Scrutari.Client.Ui.filterChange = function (client) {
    var apiFilter = Scrutari.Client.Ui.checkFilter(client);
    var apiMeta = client._apiMeta;
    var filterFicheCount = apiFilter.getFilterFicheCount(apiMeta);
    var globalFicheCount = apiMeta.getGlobalFicheCount();
    var $element = Scrutari.Client.Ui.setCount("#filterFicheCount", filterFicheCount, client._apiConfig.getLangUi());
    if (filterFicheCount === globalFicheCount) {
        $element.removeClass("scrutari-stats-Filter").removeClass("scrutari-stats-None");
        Scrutari.Client.Ui.hide("#filterFicheCount");
    } else if (filterFicheCount === 0) {
        $element.removeClass("scrutari-stats-Filter").addClass("scrutari-stats-None");
        Scrutari.Client.Ui.show("#filterFicheCount");
    } else {
        $element.addClass("scrutari-stats-Filter").removeClass("scrutari-stats-None");
        Scrutari.Client.Ui.show("#filterFicheCount");
    }
};
Scrutari.Client.Ui.checkFilter = function (client) {
    var _testEnable = function (selector) {
        return ($(selector).data("scrutariState") === "on");
    };
    var apiFilter = new Scrutari.Filter();
    if (client.withCorpus) {
        apiFilter.setWithCorpus(_testEnable("#corpusEnableButton"));
    } else {
        apiFilter.setWithBase(_testEnable("#baseEnableButton"));
    }
    apiFilter.setWithLang(_testEnable("#langEnableButton"));
    apiFilter.setWithCategory(_testEnable("#categoryEnableButton"));
    var $baseElements = $("input[name='base']:checked");
    for(var i = 0; i < $baseElements.length; i++) {
        apiFilter.addBase(parseInt($baseElements[i].value));
    }
    var $langElements = $("input[name='lang']:checked");
    for(var i = 0; i < $langElements.length; i++) {
        apiFilter.addLang($langElements[i].value);
    }
    var $categoryElements = $("input[name='category']:checked");
    for(var i = 0; i < $categoryElements.length; i++) {
        apiFilter.addCategory($categoryElements[i].value);
    }
    var $corpusElements = $("input[name='corpus']:checked");
    for(var i = 0; i < $corpusElements.length; i++) {
        apiFilter.addCorpus(parseInt($corpusElements[i].value));
    }
    return apiFilter;
};
Scrutari.Client.Ui.checkRequestParameters = function (client) {
    var apiFilter = Scrutari.Client.Ui.checkFilter(client);
    var requestParameters = new Object();
    apiFilter.checkRequestParameters(client._apiMeta, requestParameters);
    requestParameters["q-mode"] = $("input[name='q-mode']:checked").val();
    var ponderation = $("input[name='ponderation']:checked").val();
    if (ponderation === 'date') {
        requestParameters.ponderation = '15,80,5';
    }
    var periode = $.trim($("input[name='periode']").val());
    if (periode) {
        requestParameters["flt-date"] = periode;
    }
    if (Scrutari.exists("input[name='wildchar']:checked")) {
        requestParameters.wildchar = "end";
    } else {
        requestParameters.wildchar = "none";
    }
    return requestParameters;
};
Scrutari.Client.Ui.init = function (client) {
    var apiMeta = client._apiMeta;
    Scrutari.Client.Ui.initHtml(client);
    Scrutari.Client.Ui.initForms(client);
    Scrutari.Client.Ui.initButtons(client);
    Scrutari.Client.Ui.initMainTitle(client);
    Scrutari.Client.Ui.setCount("#globalFicheCount", apiMeta.getGlobalFicheCount(), client._apiConfig.getLangUi());
    Scrutari.Client.Ui.show("#globalFicheCount");
    Scrutari.Client.Ui.show("#parametersDisplayButton");
    var langArray = apiMeta.getLangArray(Scrutari.Utils.sortLangByFicheCountDesc);
    if ((langArray.length > 1) && (Scrutari.exists("#langPanel"))) {
        Scrutari.Utils.divideIntoColumns(langArray, ".scrutari-column-Lang", client.getTemplate("lang"));
            Scrutari.Client.Ui.show("#langPanel");
            $("input[name='lang']").click(function () {
                var button = $('#langEnableButton');
                if (Scrutari.Utils.toggle.getState(button, 'scrutariState') === 'off') {
                    button.click();
                    $(this).focus();
                } else {
                    Scrutari.Client.Ui.filterChange(client);
                }
            });
    }
    if ((apiMeta.withCategory()) && (Scrutari.exists("#categoryPanel"))) {
        var categoryArray = apiMeta.getCategoryArray(Scrutari.Utils.sortCategoryByRankDesc);
        Scrutari.Utils.divideIntoColumns(categoryArray, ".scrutari-column-Category", client.getTemplate("category"));
        Scrutari.Client.Ui.show("#categoryPanel");
        $("input[name='category']").click(function () {
            var button = $('#categoryEnableButton');
            if (Scrutari.Utils.toggle.getState(button, 'scrutariState') === 'off') {
                button.click();
                $(this).focus();
            } else {
                Scrutari.Client.Ui.filterChange(client);
            }
        });
    }
    if (client.withCorpus) {
        if (Scrutari.exists("#corpusPanel")) {
            var corpusSortFunction = null;
            if (client.corpusSort === "fiche-count") {
                corpusSortFunction = Scrutari.Utils.sortCorpusByFicheCountDesc;
            }
            var corpusArray =  apiMeta.getCorpusArray(corpusSortFunction);
            if (corpusArray.length > 1) {
                Scrutari.Utils.divideIntoColumns(corpusArray, ".scrutari-column-Corpus", client.getTemplate("corpus"));
                Scrutari.Client.Ui.show("#corpusPanel");
                $("input[name='corpus']").click(function () {
                    var button = $('#corpusEnableButton');
                    if (Scrutari.Utils.toggle.getState(button, 'scrutariState') === 'off') {
                        button.click();
                        $(this).focus();
                    } else {
                        Scrutari.Client.Ui.filterChange(client);
                    }
                });
            }
        }
    } else {
        if (Scrutari.exists("#basePanel")) {
            var baseSortFunction = null;
            if (client.baseSort === "fiche-count") {
                baseSortFunction = Scrutari.Utils.sortBaseByFicheCountDesc;
            }
            var baseArray =  apiMeta.getBaseArray(baseSortFunction);
            if (baseArray.length > 1) {
                Scrutari.Utils.divideIntoColumns(baseArray, ".scrutari-column-Base", client.getTemplate("base"));
                Scrutari.Client.Ui.show("#basePanel");
                $("input[name='base']").click(function () {
                    var button = $('#baseEnableButton');
                    if (Scrutari.Utils.toggle.getState(button, 'scrutariState') === 'off') {
                        button.click();
                        $(this).focus();
                    } else {
                        Scrutari.Client.Ui.filterChange(client);
                    }
                });
            }
        }
    }
    var initialQuery = client.initialQuery;
    var initialQId = client.initialQId;
    if (initialQuery.length > 0) {
        $mainSearchForm = $("#mainsearchForm");
        if (Scrutari.exists($mainSearchForm)) {
            $mainSearchForm.find("input[name='q']").val(initialQuery);
            $mainSearchForm.submit();
        }
    } else if ((initialQId) && (initialQId.length > 0)) {
        var requestParameters = Scrutari.Client.Ui.checkRequestParameters(client);
        requestParameters["qid"] = initialQId;
        var _mainsearchApiResultCallback = function (apiResult) {
            Scrutari.Client.Result.show(client, apiResult, "mainsearch");
            $("#loadingModal").modal('hide');
            $parametersDisplayButton = $('#parametersDisplayButton');
            if ($parametersDisplayButton.data("scrutariState") === "on") {
                $parametersDisplayButton.click();
            }
        };
        var _mainScrutariErrorCallback = function (error) {
            Scrutari.Client.Result.displayError(client, error, "mainsearch");
            $("#loadingModal").modal('hide');
        };
        $("#loadingModal").modal('show');
        Scrutari.Result.newSearch(client._apiConfig, requestParameters, _mainsearchApiResultCallback, _mainScrutariErrorCallback);
    }
};
Scrutari.Client.Ui.initHtml = function (client) {
    if (!client.clientSelector) {
        Scrutari.log("client.clientSelector is undefined");
        return;
    }
    var htmlObject = client.htmlObject;
    if  ((!htmlObject) || (!htmlObject.list) || (!htmlObject.list.length)) {
        Scrutari.log("client.htmlObject.list is undefined");
        return;
    }
    var _hide = function (key, text) {
        var key_start = "<!-- " + key + " -->";
        var key_end = "<!-- /" + key + " -->";
        var start_index = text.indexOf(key_start);
        var end_index = text.indexOf(key_end);
        if ((start_index > -1) && (end_index > -1) && (start_index < end_index)) {
            text = text.substring(0, start_index) + text.substring(end_index + key_end.length);
        }
        return text;
    };
    var html = "";
    for(var i= 0, len = htmlObject.list.length; i < len; i++) {
        var key = htmlObject.list[i];
        if (htmlObject.hasOwnProperty(key)) {
            html += htmlObject[key];
        }
    }
    if (client.hiddenList) {
        for(var i =0, len = client.hiddenList.length; i < len ; i++) {
            html = _hide(client.hiddenList[i], html);
        }
    }
    if (client.withCorpus) {
         html = _hide("base-panel", html);
    } else {
         html = _hide("corpus-panel", html);
    }
    if (!client.permalinkPattern) {
        html = _hide("share", html);
    }
    html = html.replace(/_ [-_\.a-z0-9]+/g, function(match) {
        return client.loc(match);
    });
    Scrutari.convert(client.clientSelector).html(html);
};
Scrutari.Client.Ui.initForms = function (client) {
    $("#mainsearchForm").submit(function () {
        var q = $(this).find("input[name='q']").val();
        q = $.trim(q);
        if (q.length > 0) {
            var requestParameters = Scrutari.Client.Ui.checkRequestParameters(client);
            requestParameters["log"] = "all";
            requestParameters["q"] = q;
            var _mainsearchApiResultCallback = function (apiResult) {
                Scrutari.Client.Result.show(client, apiResult, "mainsearch");
                $("#loadingModal").modal('hide');
                $parametersDisplayButton = $('#parametersDisplayButton');
                if ($parametersDisplayButton.data("scrutariState") === "on") {
                    $parametersDisplayButton.click();
                }
            };
            var _mainScrutariErrorCallback = function (error) {
                Scrutari.Client.Result.displayError(client, error, "mainsearch");
                $("#loadingModal").modal('hide');
            };
            $("#loadingModal").modal('show');
            Scrutari.Result.newSearch(client._apiConfig, requestParameters, _mainsearchApiResultCallback, _mainScrutariErrorCallback);
        }
        return false;
    });
    $("#subsearchForm").submit(function () {
        var q = $(this).find("input[name='q']").val();
        q = $.trim(q);
        if ((q.length > 0) && (client._currentApiResult)) {
            var requestParameters = Scrutari.Client.Ui.checkRequestParameters(client);
            requestParameters["q"] = q;
            requestParameters["flt-qid"] = client._currentApiResult.getQId();
            var _subsearchApiResultCallback = function (apiResult) {
                Scrutari.Client.Result.show(client, apiResult, "subsearch");
                $("#loadingModal").modal('hide');
            };
            var _subsearchScrutariErrorCallback = function (error) {
                Scrutari.Client.Result.displayError(client, error, "subsearch");
                $("#loadingModal").modal('hide');
            };
            $("#loadingModal").modal('show');
            Scrutari.Result.newSearch(client._apiConfig, requestParameters, _subsearchApiResultCallback, _subsearchScrutariErrorCallback);
        }
        return false;
    });
};
Scrutari.Client.Ui.initButtons = function (client) {
    $('#langEnableButton').click(function () {
        var $this = $(this);
        var state = Scrutari.Utils.toggle($this, "scrutariState");
        Scrutari.Utils.toggle.disabled("#langCheckAllButton, #langUncheckAllButton", state);
        Scrutari.Utils.toggle.classes(".scrutari-column-Lang", state, "", "scrutari-Disabled");
        Scrutari.Utils.toggle.text($this.children("span"), "scrutariAlternate");
        var $langFilterLabel = Scrutari.Utils.toggle.text("#langFilterLabel", "scrutariAlternate");
        Scrutari.Utils.toggle.classes($langFilterLabel, state, "scrutari-panel-Active", "scrutari-Disabled");
        Scrutari.Client.Ui.filterChange(client);
    });
    $('#langCheckAllButton').click(function () {
        Scrutari.Utils.check("input[name='lang']");
        Scrutari.Client.Ui.filterChange(client);
    });
    $('#langUncheckAllButton').click(function () {
        Scrutari.Utils.uncheck("input[name='lang']");
        Scrutari.Client.Ui.filterChange(client);
    });
    $('#categoryEnableButton').click(function () {
        var $this = $(this);
        var state = Scrutari.Utils.toggle($this, "scrutariState");
        Scrutari.Utils.toggle.disabled("#categoryCheckAllButton, #categoryUncheckAllButton", state);
        Scrutari.Utils.toggle.classes(".scrutari-column-Category", state, "", "scrutari-Disabled");
        Scrutari.Utils.toggle.text($this.children("span"), "scrutariAlternate");
        var $categoryFilterLabel = Scrutari.Utils.toggle.text("#categoryFilterLabel", "scrutariAlternate");
        Scrutari.Utils.toggle.classes($categoryFilterLabel, state, "scrutari-panel-Active", "scrutari-Disabled");
        Scrutari.Client.Ui.filterChange(client);
    });
    $('#categoryCheckAllButton').click(function () {
        Scrutari.Utils.check("input[name='category']");
        Scrutari.Client.Ui.filterChange(client);
    });
    $('#categoryUncheckAllButton').click(function () {
        Scrutari.Utils.uncheck("input[name='category']");
        Scrutari.Client.Ui.filterChange(client);
    });
    if (client.withCorpus) {
        $('#corpusEnableButton').click(function () {
            var $this = $(this);
            var state = Scrutari.Utils.toggle($this, "scrutariState");
            Scrutari.Utils.toggle.disabled("#corpusCheckAllButton, #corpusUncheckAllButton", state);
            Scrutari.Utils.toggle.classes(".scrutari-column-Corpus", state, "", "scrutari-Disabled");
            Scrutari.Utils.toggle.text($this.children("span"), "scrutariAlternate");
            var $corpusFilterLabel = Scrutari.Utils.toggle.text("#corpusFilterLabel", "scrutariAlternate");
            Scrutari.Utils.toggle.classes($corpusFilterLabel, state, "scrutari-panel-Active", "scrutari-Disabled");
            Scrutari.Client.Ui.filterChange(client);
        });
        $('#corpusCheckAllButton').click(function () {
            Scrutari.Utils.check("input[name='corpus']");
            Scrutari.Client.Ui.filterChange(client);
        });
        $('#corpusUncheckAllButton').click(function () {
            Scrutari.Utils.uncheck("input[name='corpus']");
            Scrutari.Client.Ui.filterChange(client);
        });
    } else {
        $('#baseEnableButton').click(function () {
            var $this = $(this);
            var state = Scrutari.Utils.toggle($this, "scrutariState");
            Scrutari.Utils.toggle.disabled("#baseCheckAllButton, #baseUncheckAllButton", state);
            Scrutari.Utils.toggle.classes(".scrutari-column-Base", state, "", "scrutari-Disabled");
            Scrutari.Utils.toggle.text($this.children("span"), "scrutariAlternate");
            var $baseFilterLabel = Scrutari.Utils.toggle.text("#baseFilterLabel", "scrutariAlternate");
            Scrutari.Utils.toggle.classes($baseFilterLabel, state, "scrutari-panel-Active", "scrutari-Disabled");
            Scrutari.Client.Ui.filterChange(client);
        });
        $('#baseCheckAllButton').click(function () {
            Scrutari.Utils.check("input[name='base']");
            Scrutari.Client.Ui.filterChange(client);
        });
        $('#baseUncheckAllButton').click(function () {
            Scrutari.Utils.uncheck("input[name='base']");
            Scrutari.Client.Ui.filterChange(client);
        });
    }
    $('#parametersDisplayButton').click(function () {
        var state = Scrutari.Utils.toggle($(this), "scrutariState");
        if (state === 'on') {
            Scrutari.Client.Ui.show('#parametersArea');
        } else {
            Scrutari.Client.Ui.hide('#parametersArea');
        }
    });
    $('#modeHelpButton').click(function () {
        $modeHelpModal = $("#modeHelpModal");
        $modalBody = $modeHelpModal.find(".modal-body");
        if ($modalBody.children().length === 0) {
            $modalBody.html(client.loc("_ help_mode.html"));
        }
        $modeHelpModal.modal('show');
    });
    $('#ponderationHelpButton').click(function () {
        $ponderationHelpModal = $("#ponderationHelpModal");
        $modalBody = $ponderationHelpModal.find(".modal-body");
        if ($modalBody.children().length === 0) {
            $modalBody.html(client.loc("_ help_ponderation.html"));
        }
        $ponderationHelpModal.modal('show');
    });
    $('#periodeHelpButton').click(function () {
        $periodeHelpModal = $("#periodeHelpModal");
        $modalBody = $periodeHelpModal.find(".modal-body");
        if ($modalBody.children().length === 0) {
            $modalBody.html(client.loc("_ help_periode.html"));
        }
        $periodeHelpModal.modal('show');
    });
    $("input[name='q-mode']").click(function () {
        if (this.value === 'operation') {
            Scrutari.Utils.disable("input[name='wildchar']");
        } else {
            Scrutari.Utils.enable("input[name='wildchar']");
        }
    });
};
Scrutari.Client.Ui.initMainTitle = function (client) {
    var html = "";
    if (client.mainTitle) {
        if ($.isFunction(client.mainTitle)) {
            html = client.mainTitle(client);
        } else {
            html = client.mainTitle;
        }
    } else {
        html += client.loc('_ title_main');
        html += " – ";
        var title = client._apiMeta.getTitle();
        if (title) {
            html += Scrutari.escape(title);
        } else {
            html += "[";
            html += client._apiConfig.getName();
            html += "]";
        }
    }
    $("#mainTitle").html(html);
};
Scrutari.Client.FrameMode = {};
Scrutari.Client.FrameMode.is = function (client) {
    return (client.mode === "frame");
};
Scrutari.Client.FrameMode.setCurrentFicheBlock = function (code, scroll) {
    $(".scrutari-fiche-CurrentBlock").removeClass("scrutari-fiche-CurrentBlock");
    var $ficheBlock = $("#ficheBlock_" + code);
    $ficheBlock.addClass("scrutari-fiche-CurrentBlock");
    if (scroll) {
        var top = $ficheBlock.offset().top;
        var $window =  $(window);
        var windowTop = $window.scrollTop();
        var windowBottom = windowTop + $window.height();
        if ((top < windowTop) || (top > (windowBottom - 20))) {
            if ($window.height() > 300) {
                top = top - 150;
            }
            $window.scrollTop(top);
        }
    }
};
Scrutari.Client.FrameMode.clear = function () {
    window.parent.frames["Navigation"].clear();
};
Scrutari.Client.FrameMode.changePlageCallback = function (plageFicheArray, changePlageArguments) {
    var _linkClick = function () {
        var idx = this.id.indexOf('_');
        var code = this.id.substring(idx + 1);
        Scrutari.Client.FrameMode.setCurrentFicheBlock(code, false);
        window.parent.frames["Navigation"].setCurrentFiche(code, plageFicheArray);
    };
    Scrutari.Client.FrameMode.clear();
    if (changePlageArguments.categoryName) {
        $("#categoryFiches_" + changePlageArguments.categoryName + " a.scrutari-fiche-Link").click(_linkClick);
    } else {
        $("a.scrutari-fiche-Link").click(_linkClick);
    }
};
